// Copyright (C) 2017 Free Software Foundation, Inc.
// Contributed by Marcel Achim (rb2501c@freescale.com)

// This file is part of GCC.

// GCC is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published
// by the Free Software Foundation; either version 3, or (at your
// option) any later version.

// GCC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details.

// You should have received a copy of the GNU General Public License
// along with GCC; see the file COPYING3.  If not see
// <http://www.gnu.org/licenses/>.

// lsp.h
// Lightweight Signal Processing (LSP) APU Instrinsics

#ifndef _LSP_H
#define _LSP_H

#if defined(__ppc) && defined(__LSP_APU__)

#include <stdint.h>
#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

#if defined(__STDC__) || defined(__PROTOTYPES__) || defined(__EDG__)
#define __ARGS(x) x
#else
#define __ARGS(x) ()
#endif

#if defined(__ATTRIBUTES)
#define __ATTRIBUTE(x) __attribute__(x)
#else
#define __ATTRIBUTE(x)
#endif

#if defined(__cplusplus)
#define __Inline_keyword_	inline
#elif defined(__GNUC__)
#define __Inline_keyword_	static inline
#else
#define __Inline_keyword_	__inline
#endif

///////////////////////////////////////////////////////////////////////////////

// LSP Opaque Data Types
typedef uint16_t __lsp16_16__;    // 16 bit field with one half word
typedef uint32_t __lsp32_32__;    // 32 bit field with one word
typedef uint32_t __lsp32_16__;    // 32 bit field with two half words
typedef uint64_t __lsp64_64__;    // 64 bit field with one double word
typedef uint64_t __lsp64_32__;    // 64 bit field with two words

// LSP Data Types
typedef __lsp16_16__ __lsp16_s16__;   // 16 bit signed with one half word
typedef __lsp32_32__ __lsp32_s32__;   // 32 bit signed with one word
typedef __lsp32_16__ __lsp32_s16__;   // 32 bit signed with two half words
typedef __lsp64_64__ __lsp64_s64__;   // 64 bit signed with one double word
typedef __lsp64_32__ __lsp64_s32__;   // 64 bit signed with two words
typedef __lsp16_16__ __lsp16_u16__;   // 16 bit unsigned with one half word
typedef __lsp32_32__ __lsp32_u32__;   // 32 bit unsigned with one word
typedef __lsp32_16__ __lsp32_u16__;   // 32 bit unsigned with two half words
typedef __lsp64_64__ __lsp64_u64__;   // 64 bit unsigned with one double word
typedef __lsp64_32__ __lsp64_u32__;   // 64 bit unsigned with two words
typedef __lsp16_16__ __lsp16_uf16__;  // unsigned fraction      [ 0.16 format]
typedef __lsp32_32__ __lsp32_uf32__;  // unsigned fraction      [ 0.32 format]
typedef __lsp32_16__ __lsp32_uf16__;  // two unsigned fractions [ 0.16 format]
typedef __lsp16_16__ __lsp16_sf16__;  // signed fraction        [ 0.16 format]
typedef __lsp32_32__ __lsp32_sf32__;  // signed fraction        [ 1.31 format]
typedef __lsp32_16__ __lsp32_sf16__;  // two signed fractions   [ 1.15 format]
typedef __lsp32_32__ __lsp32_9_23__;  // signed fraction        [ 9.23 format]
typedef __lsp64_32__ __lsp64_9_23__;  // two signed fractions   [ 9.23 format]
typedef __lsp64_64__ __lsp64_33_31__; // signed fraction        [33.31 format]
typedef __lsp64_64__ __lsp64_17_47__; // signed fraction        [17.47 format]

// LSP Immediate Types
typedef uint8_t __lsp5_uimm__;    // Unsigned 5-bit integer
typedef uint8_t __lsp4_uimm__;    // Unsigned 4-bit integer
typedef  int8_t __lsp5_simm__;    // Signed 5-bit integer
typedef uint8_t __lsp2_offset__;  // 2-bit byte offset, range of [1:3]

// LSP Condition Register Field
typedef uint32_t __lsp4_crd__;    // 4-bit condition register field

// LSP Comparison Bit Select
// These are passed to the __zvcmp* intrinsics to select which bits of the
// condition register after the comparison should be checked.
typedef uint8_t __lsp3_cmpsel__;  // 3-bit comparison bit select field
#define __LSP_CMPSEL_HIGH__ 0	  // Select high (cmp) high bit
#define __LSP_CMPSEL_LOW__  1     // Select low  (cmp) low bit
#define __LSP_CMPSEL_OR__   2     // Select high (cmp) high | low  (cmp) low bit
#define __LSP_CMPSEL_AND__  3     // Select high (cmp) high & low  (cmp) low bit
#define __LSP_CMPSEL_ALL__  4     // All four bits in the condition register

// LSP Halfword Select Comparison Modes
// These are passed to the __zvselh intrinsic to select which comparison
// instruction should be generated before the zvsel instruction.
typedef uint8_t __lsp2_selmode__; // 3-bit comparison zvsel comparison mode
#define __LSP_ZVSEL_GTS__   0     // zvcmpgths cr0, rA, rB; zvselh rA, rB
#define __LSP_ZVSEL_GTU__   1     // zvcmpgthu cr0, rA, rB; zvselh rA, rB
#define __LSP_ZVSEL_LTS__   2     // zvcmplths cr0, rA, rB; zvselh rA, rB
#define __LSP_ZVSEL_LTU__   3     // zvcmplthu cr0, rA, rB; zvselh rA, rB

///////////////////////////////////////////////////////////////////////////////

// LSP vector type creation
__Inline_keyword_ __lsp64_32__ __lsp_create_64_32(__lsp32_32__ upper, __lsp32_32__ lower)
    { return ((((__lsp64_32__)upper) << 32) | (((__lsp64_32__)lower) & 0xFFFFFFFF)); }
__Inline_keyword_ __lsp32_16__ __lsp_create_32_16(__lsp16_16__ upper, __lsp16_16__ lower)
    { return ((((__lsp32_16__)upper) << 16) | (((__lsp32_16__)lower) & 0x0000FFFF)); }

// LSP vector type extraction
__Inline_keyword_ __lsp32_32__ __lsp_extract_upper_word(__lsp64_32__ x)  { return (__lsp32_32__)(x >> 32); }
__Inline_keyword_ __lsp32_32__ __lsp_extract_lower_word(__lsp64_32__ x)  { return (__lsp32_32__)(x);       }
__Inline_keyword_ __lsp16_16__ __lsp_extract_upper_hword(__lsp32_16__ x) { return (__lsp16_16__)(x >> 16); }
__Inline_keyword_ __lsp16_16__ __lsp_extract_lower_hword(__lsp32_16__ x) { return (__lsp16_16__)(x);       }

///////////////////////////////////////////////////////////////////////////////

// SPEFSCR access intrinsics
#define __lsp_get_spefscr_sov()  (__MFSPR(512) & (1 << 15))
    // unsigned int __lsp_get_spefscr_sov(void);
    // Returns the SOV bit of the SPEFSCR
#define __lsp_get_spefscr_ov()   (__MFSPR(512) & (1 << 14))
    // unsigned int __lsp_get_spefscr_ov(void);
    // Returns the OV bit of the SPEFSCR
#define __lsp_clr_spefscr_sov()  (__MTSPR(512, __MFSPR(512) & ~(1 << 15)))
    // void __lsp_clr_spefscr_sov(void);
    // Clears the SOV bit of the SPEFSCR

///////////////////////////////////////////////////////////////////////////////

// LSP circular buffer increment
__lsp32_32__ __lsp_modify_circ(__lsp32_32__ a);

///////////////////////////////////////////////////////////////////////////////

// LSP Instruction Instrinsics
#define __zvaddih				__builtin_lsp_zvaddih
#define __zvsubifh				__builtin_lsp_zvsubifh
#define __zvaddh				__builtin_lsp_zvaddh
#define __zvsubfh				__builtin_lsp_zvsubfh
#define __zvaddsubfh				__builtin_lsp_zvaddsubfh
#define __zvsubfaddh				__builtin_lsp_zvsubfaddh
#define __zvaddhx				__builtin_lsp_zvaddhx
#define __zvsubfhx				__builtin_lsp_zvsubfhx
#define __zvaddsubfhx				__builtin_lsp_zvaddsubfhx
#define __zvsubfaddhx				__builtin_lsp_zvsubfaddhx
#define __zvaddhus				__builtin_lsp_zvaddhus
#define __zvsubfhus				__builtin_lsp_zvsubfhus
#define __zvaddhss				__builtin_lsp_zvaddhss
#define __zvsubfhss				__builtin_lsp_zvsubfhss
#define __zvaddsubfhss				__builtin_lsp_zvaddsubfhss
#define __zvsubfaddhss				__builtin_lsp_zvsubfaddhss
#define __zvaddhxss				__builtin_lsp_zvaddhxss
#define __zvsubfhxss				__builtin_lsp_zvsubfhxss
#define __zvaddsubfhxss				__builtin_lsp_zvaddsubfhxss
#define __zvsubfaddhxss				__builtin_lsp_zvsubfaddhxss
#define __zvmergehih				__builtin_lsp_zvmergehih
#define __zvmergeloh				__builtin_lsp_zvmergeloh
#define __zvmergehiloh				__builtin_lsp_zvmergehiloh
#define __zvmergelohih				__builtin_lsp_zvmergelohih
#define __zvpkshgwshfrs				__builtin_lsp_zvpkshgwshfrs
#define __zvpkswshfrs				__builtin_lsp_zvpkswshfrs
#define __zvpkswuhs				__builtin_lsp_zvpkswuhs
#define __zvpkswshs				__builtin_lsp_zvpkswshs
#define __zvpkuwuhs				__builtin_lsp_zvpkuwuhs
#define __zvsplatih				__builtin_lsp_zvsplatih
#define __zvsplatfih				__builtin_lsp_zvsplatfih
#define __zcntlsw				__builtin_lsp_zcntlsw
#define __zvcntlzh				__builtin_lsp_zvcntlzh
#define __zvcntlsh				__builtin_lsp_zvcntlsh
#define __znegws				__builtin_lsp_znegws
#define __zvnegh				__builtin_lsp_zvnegh
#define __zvneghs				__builtin_lsp_zvneghs
#define __zvnegho				__builtin_lsp_zvnegho
#define __zvneghos				__builtin_lsp_zvneghos
#define __zrndwh				__builtin_lsp_zrndwh
#define __zrndwhss				__builtin_lsp_zrndwhss
#define __zvabsh				__builtin_lsp_zvabsh
#define __zvabshs				__builtin_lsp_zvabshs
#define __zabsw					__builtin_lsp_zabsw
#define __zabsws				__builtin_lsp_zabsws
#define __zsatswuw				__builtin_lsp_zsatswuw
#define __zsatuwsw				__builtin_lsp_zsatuwsw
#define __zsatswuh				__builtin_lsp_zsatswuh
#define __zsatswsh				__builtin_lsp_zsatswsh
#define __zvsatshuh				__builtin_lsp_zvsatshuh
#define __zvsatuhsh				__builtin_lsp_zvsatuhsh
#define __zsatuwuh				__builtin_lsp_zsatuwuh
#define __zsatuwsh				__builtin_lsp_zsatuwsh
#define __zsatsduw				__builtin_lsp_zsatsduw
#define __zsatsdsw				__builtin_lsp_zsatsdsw
#define __zsatuduw				__builtin_lsp_zsatuduw
#define __zdivwsf				__builtin_lsp_zdivwsf
#define __zvsrhu				__builtin_lsp_zvsrhu
#define __zvsrhs				__builtin_lsp_zvsrhs
#define __zvsrhiu				__builtin_lsp_zvsrhiu
#define __zvsrhis				__builtin_lsp_zvsrhis
#define __zvslh					__builtin_lsp_zvslh
#define __zvrlh					__builtin_lsp_zvrlh
#define __zvslhi				__builtin_lsp_zvslhi
#define __zvrlhi				__builtin_lsp_zvrlhi
#define __zvslhus				__builtin_lsp_zvslhus
#define __zvslhss				__builtin_lsp_zvslhss
#define __zvslhius				__builtin_lsp_zvslhius
#define __zvslhiss				__builtin_lsp_zvslhiss
#define __zslwus				__builtin_lsp_zslwus
#define __zslwss				__builtin_lsp_zslwss
#define __zslwius				__builtin_lsp_zslwius
#define __zslwiss				__builtin_lsp_zslwiss
#define __zaddwgui				__builtin_lsp_zaddwgui
#define __zsubfwgui				__builtin_lsp_zsubfwgui
#define __zaddwgsi				__builtin_lsp_zaddwgsi
#define __zsubfwgsi				__builtin_lsp_zsubfwgsi
#define __zaddwgsf				__builtin_lsp_zaddwgsf
#define __zsubfwgsf				__builtin_lsp_zsubfwgsf
#define __zaddd					__builtin_lsp_zaddd
#define __zsubfd				__builtin_lsp_zsubfd
#define __zadddss				__builtin_lsp_zadddss
#define __zsubfdss				__builtin_lsp_zsubfdss
#define __zadddus				__builtin_lsp_zadddus
#define __zsubfdus				__builtin_lsp_zsubfdus
#define __zvaddsubfw				__builtin_lsp_zvaddsubfw
#define __zvsubfaddw				__builtin_lsp_zvsubfaddw
#define __zvaddw				__builtin_lsp_zvaddw
#define __zvsubfw				__builtin_lsp_zvsubfw
#define __zvaddsubfwss				__builtin_lsp_zvaddsubfwss
#define __zvsubfaddwss				__builtin_lsp_zvsubfaddwss
#define __zvaddwss				__builtin_lsp_zvaddwss
#define __zvsubfwss				__builtin_lsp_zvsubfwss
#define __zvaddwus				__builtin_lsp_zvaddwus
#define __zvsubfwus				__builtin_lsp_zvsubfwus
#define __zvunpkhgwsf				__builtin_lsp_zvunpkhgwsf
#define __zvunpkhsf				__builtin_lsp_zvunpkhsf
#define __zvunpkhui				__builtin_lsp_zvunpkhui
#define __zvunpkhsi				__builtin_lsp_zvunpkhsi
#define __zunpkwgsf				__builtin_lsp_zunpkwgsf
#define __zvdotphgwasmf				__builtin_lsp_zvdotphgwasmf
#define __zvdotphgwasmfr			__builtin_lsp_zvdotphgwasmfr
#define __zvdotphxgwasmf			__builtin_lsp_zvdotphxgwasmf
#define __zvdotphxgwasmfr			__builtin_lsp_zvdotphxgwasmfr
#define __zvdotphgwssmf				__builtin_lsp_zvdotphgwssmf
#define __zvdotphgwssmfr			__builtin_lsp_zvdotphgwssmfr
#define __zvdotphgwasmfaa			__builtin_lsp_zvdotphgwasmfaa
#define __zvdotphgwasmfraa			__builtin_lsp_zvdotphgwasmfraa
#define __zvdotphgwasmfan			__builtin_lsp_zvdotphgwasmfan
#define __zvdotphgwasmfran			__builtin_lsp_zvdotphgwasmfran
#define __zvdotphxgwasmfaa			__builtin_lsp_zvdotphxgwasmfaa
#define __zvdotphxgwasmfraa			__builtin_lsp_zvdotphxgwasmfraa
#define __zvdotphxgwasmfan			__builtin_lsp_zvdotphxgwasmfan
#define __zvdotphxgwasmfran			__builtin_lsp_zvdotphxgwasmfran
#define __zvdotphgwssmfaa			__builtin_lsp_zvdotphgwssmfaa
#define __zvdotphgwssmfraa			__builtin_lsp_zvdotphgwssmfraa
#define __zvdotphgwssmfan			__builtin_lsp_zvdotphgwssmfan
#define __zvdotphgwssmfran			__builtin_lsp_zvdotphgwssmfran
#define __zvmhulgwsmf				__builtin_lsp_zvmhulgwsmf
#define __zvmhulgwsmfr				__builtin_lsp_zvmhulgwsmfr
#define __zvmhllgwsmf				__builtin_lsp_zvmhllgwsmf
#define __zvmhllgwsmfr				__builtin_lsp_zvmhllgwsmfr
#define __zvmhuugwsmf				__builtin_lsp_zvmhuugwsmf
#define __zvmhuugwsmfr				__builtin_lsp_zvmhuugwsmfr
#define __zvmhxlgwsmf				__builtin_lsp_zvmhxlgwsmf
#define __zvmhxlgwsmfr				__builtin_lsp_zvmhxlgwsmfr
#define __zvmhulgwsmfaa				__builtin_lsp_zvmhulgwsmfaa
#define __zvmhulgwsmfraa			__builtin_lsp_zvmhulgwsmfraa
#define __zvmhulgwsmfan				__builtin_lsp_zvmhulgwsmfan
#define __zvmhulgwsmfran			__builtin_lsp_zvmhulgwsmfran
#define __zvmhulgwsmfanp			__builtin_lsp_zvmhulgwsmfanp
#define __zvmhulgwsmfranp			__builtin_lsp_zvmhulgwsmfranp
#define __zvmhllgwsmfaa				__builtin_lsp_zvmhllgwsmfaa
#define __zvmhllgwsmfraa			__builtin_lsp_zvmhllgwsmfraa
#define __zvmhllgwsmfan				__builtin_lsp_zvmhllgwsmfan
#define __zvmhllgwsmfran			__builtin_lsp_zvmhllgwsmfran
#define __zvmhllgwsmfanp			__builtin_lsp_zvmhllgwsmfanp
#define __zvmhllgwsmfranp			__builtin_lsp_zvmhllgwsmfranp
#define __zvmhuugwsmfaa				__builtin_lsp_zvmhuugwsmfaa
#define __zvmhuugwsmfraa			__builtin_lsp_zvmhuugwsmfraa
#define __zvmhuugwsmfan				__builtin_lsp_zvmhuugwsmfan
#define __zvmhuugwsmfran			__builtin_lsp_zvmhuugwsmfran
#define __zvmhuugwsmfanp			__builtin_lsp_zvmhuugwsmfanp
#define __zvmhuugwsmfranp			__builtin_lsp_zvmhuugwsmfranp
#define __zvmhxlgwsmfaa				__builtin_lsp_zvmhxlgwsmfaa
#define __zvmhxlgwsmfraa			__builtin_lsp_zvmhxlgwsmfraa
#define __zvmhxlgwsmfan				__builtin_lsp_zvmhxlgwsmfan
#define __zvmhxlgwsmfran			__builtin_lsp_zvmhxlgwsmfran
#define __zvmhxlgwsmfanp			__builtin_lsp_zvmhxlgwsmfanp
#define __zvmhxlgwsmfranp			__builtin_lsp_zvmhxlgwsmfranp
#define __zmheogwsmf				__builtin_lsp_zmheogwsmf
#define __zmheogwsmfr				__builtin_lsp_zmheogwsmfr
#define __zmhegwsmf				__builtin_lsp_zmhegwsmf
#define __zmhegwsmfr				__builtin_lsp_zmhegwsmfr
#define __zmhogwsmf				__builtin_lsp_zmhogwsmf
#define __zmhogwsmfr				__builtin_lsp_zmhogwsmfr
#define __zmheogwsmfaa				__builtin_lsp_zmheogwsmfaa
#define __zmheogwsmfraa				__builtin_lsp_zmheogwsmfraa
#define __zmheogwsmfan				__builtin_lsp_zmheogwsmfan
#define __zmheogwsmfran				__builtin_lsp_zmheogwsmfran
#define __zmhegwsmfaa				__builtin_lsp_zmhegwsmfaa
#define __zmhegwsmfraa				__builtin_lsp_zmhegwsmfraa
#define __zmhegwsmfan				__builtin_lsp_zmhegwsmfan
#define __zmhegwsmfran				__builtin_lsp_zmhegwsmfran
#define __zmhogwsmfaa				__builtin_lsp_zmhogwsmfaa
#define __zmhogwsmfraa				__builtin_lsp_zmhogwsmfraa
#define __zmhogwsmfan				__builtin_lsp_zmhogwsmfan
#define __zmhogwsmfran				__builtin_lsp_zmhogwsmfran
#define __zmhegui				__builtin_lsp_zmhegui
#define __zmhegsi				__builtin_lsp_zmhegsi
#define __zmhegsui				__builtin_lsp_zmhegsui
#define __zmhegsmf				__builtin_lsp_zmhegsmf
#define __zmheogui				__builtin_lsp_zmheogui
#define __zmheogsi				__builtin_lsp_zmheogsi
#define __zmheogsui				__builtin_lsp_zmheogsui
#define __zmheogsmf				__builtin_lsp_zmheogsmf
#define __zmhogui				__builtin_lsp_zmhogui
#define __zmhogsi				__builtin_lsp_zmhogsi
#define __zmhogsui				__builtin_lsp_zmhogsui
#define __zmhogsmf				__builtin_lsp_zmhogsmf
#define __zvdotphgaui				__builtin_lsp_zvdotphgaui
#define __zvdotphgasi				__builtin_lsp_zvdotphgasi
#define __zvdotphgasui				__builtin_lsp_zvdotphgasui
#define __zvdotphgasmf				__builtin_lsp_zvdotphgasmf
#define __zvdotphxgaui				__builtin_lsp_zvdotphxgaui
#define __zvdotphxgasi				__builtin_lsp_zvdotphxgasi
#define __zvdotphxgasui				__builtin_lsp_zvdotphxgasui
#define __zvdotphxgasmf				__builtin_lsp_zvdotphxgasmf
#define __zvdotphgsui				__builtin_lsp_zvdotphgsui
#define __zvdotphgssi				__builtin_lsp_zvdotphgssi
#define __zvdotphgssui				__builtin_lsp_zvdotphgssui
#define __zvdotphgssmf				__builtin_lsp_zvdotphgssmf
#define __zmheguiaa				__builtin_lsp_zmheguiaa
#define __zvdotphgauiaa				__builtin_lsp_zvdotphgauiaa
#define __zmheguian				__builtin_lsp_zmheguian
#define __zvdotphgauian				__builtin_lsp_zvdotphgauian
#define __zmhegsiaa				__builtin_lsp_zmhegsiaa
#define __zvdotphgasiaa				__builtin_lsp_zvdotphgasiaa
#define __zmhegsian				__builtin_lsp_zmhegsian
#define __zvdotphgasian				__builtin_lsp_zvdotphgasian
#define __zmhegsuiaa				__builtin_lsp_zmhegsuiaa
#define __zvdotphgasuiaa			__builtin_lsp_zvdotphgasuiaa
#define __zmhegsuian				__builtin_lsp_zmhegsuian
#define __zvdotphgasuian			__builtin_lsp_zvdotphgasuian
#define __zmhegsmfaa				__builtin_lsp_zmhegsmfaa
#define __zvdotphgasmfaa			__builtin_lsp_zvdotphgasmfaa
#define __zmhegsmfan				__builtin_lsp_zmhegsmfan
#define __zvdotphgasmfan			__builtin_lsp_zvdotphgasmfan
#define __zmheoguiaa				__builtin_lsp_zmheoguiaa
#define __zvdotphxgauiaa			__builtin_lsp_zvdotphxgauiaa
#define __zmheoguian				__builtin_lsp_zmheoguian
#define __zvdotphxgauian			__builtin_lsp_zvdotphxgauian
#define __zmheogsiaa				__builtin_lsp_zmheogsiaa
#define __zvdotphxgasiaa			__builtin_lsp_zvdotphxgasiaa
#define __zmheogsian				__builtin_lsp_zmheogsian
#define __zvdotphxgasian			__builtin_lsp_zvdotphxgasian
#define __zmheogsuiaa				__builtin_lsp_zmheogsuiaa
#define __zvdotphxgasuiaa			__builtin_lsp_zvdotphxgasuiaa
#define __zmheogsuian				__builtin_lsp_zmheogsuian
#define __zvdotphxgasuian			__builtin_lsp_zvdotphxgasuian
#define __zmheogsmfaa				__builtin_lsp_zmheogsmfaa
#define __zvdotphxgasmfaa			__builtin_lsp_zvdotphxgasmfaa
#define __zmheogsmfan				__builtin_lsp_zmheogsmfan
#define __zvdotphxgasmfan			__builtin_lsp_zvdotphxgasmfan
#define __zmhoguiaa				__builtin_lsp_zmhoguiaa
#define __zvdotphgsuiaa				__builtin_lsp_zvdotphgsuiaa
#define __zmhoguian				__builtin_lsp_zmhoguian
#define __zvdotphgsuian				__builtin_lsp_zvdotphgsuian
#define __zmhogsiaa				__builtin_lsp_zmhogsiaa
#define __zvdotphgssiaa				__builtin_lsp_zvdotphgssiaa
#define __zmhogsian				__builtin_lsp_zmhogsian
#define __zvdotphgssian				__builtin_lsp_zvdotphgssian
#define __zmhogsuiaa				__builtin_lsp_zmhogsuiaa
#define __zvdotphgssuiaa			__builtin_lsp_zvdotphgssuiaa
#define __zmhogsuian				__builtin_lsp_zmhogsuian
#define __zvdotphgssuian			__builtin_lsp_zvdotphgssuian
#define __zmhogsmfaa				__builtin_lsp_zmhogsmfaa
#define __zvdotphgssmfaa			__builtin_lsp_zvdotphgssmfaa
#define __zmhogsmfan				__builtin_lsp_zmhogsmfan
#define __zvdotphgssmfan			__builtin_lsp_zvdotphgssmfan
#define __zmwgui				__builtin_lsp_zmwgui
#define __zmwgsi				__builtin_lsp_zmwgsi
#define __zmwgsui				__builtin_lsp_zmwgsui
#define __zmwguiaa				__builtin_lsp_zmwguiaa
#define __zmwguian				__builtin_lsp_zmwguian
#define __zmwgsiaa				__builtin_lsp_zmwgsiaa
#define __zmwgsian				__builtin_lsp_zmwgsian
#define __zmwgsuiaa				__builtin_lsp_zmwgsuiaa
#define __zmwgsuian				__builtin_lsp_zmwgsuian
#define __zmwguiaas				__builtin_lsp_zmwguiaas
#define __zmwguians				__builtin_lsp_zmwguians
#define __zmwgsiaas				__builtin_lsp_zmwgsiaas
#define __zmwgsians				__builtin_lsp_zmwgsians
#define __zmwgsuiaas				__builtin_lsp_zmwgsuiaas
#define __zmwgsuians				__builtin_lsp_zmwgsuians
#define __zmwgsmf				__builtin_lsp_zmwgsmf
#define __zmwgsmfr				__builtin_lsp_zmwgsmfr
#define __zmwgsmfaa				__builtin_lsp_zmwgsmfaa
#define __zmwgsmfraa				__builtin_lsp_zmwgsmfraa
#define __zmwgsmfan				__builtin_lsp_zmwgsmfan
#define __zmwgsmfran				__builtin_lsp_zmwgsmfran
#define __zvmhulsf				__builtin_lsp_zvmhulsf
#define __zvmhulsfr				__builtin_lsp_zvmhulsfr
#define __zvmhllsf				__builtin_lsp_zvmhllsf
#define __zvmhllsfr				__builtin_lsp_zvmhllsfr
#define __zvmhuusf				__builtin_lsp_zvmhuusf
#define __zvmhuusfr				__builtin_lsp_zvmhuusfr
#define __zvmhxlsf				__builtin_lsp_zvmhxlsf
#define __zvmhxlsfr				__builtin_lsp_zvmhxlsfr
#define __zvmhului				__builtin_lsp_zvmhului
#define __zvmhulsi				__builtin_lsp_zvmhulsi
#define __zvmhulsui				__builtin_lsp_zvmhulsui
#define __zvmhllui				__builtin_lsp_zvmhllui
#define __zvmhllsi				__builtin_lsp_zvmhllsi
#define __zvmhllsui				__builtin_lsp_zvmhllsui
#define __zvmhuuui				__builtin_lsp_zvmhuuui
#define __zvmhuusi				__builtin_lsp_zvmhuusi
#define __zvmhuusui				__builtin_lsp_zvmhuusui
#define __zvmhxlsui				__builtin_lsp_zvmhxlsui
#define __zvmhxlsi				__builtin_lsp_zvmhxlsi
#define __zvmhxlui				__builtin_lsp_zvmhxlui
#define __zvmhuluiaa				__builtin_lsp_zvmhuluiaa
#define __zvmhuluiaas				__builtin_lsp_zvmhuluiaas
#define __zvmhuluian				__builtin_lsp_zvmhuluian
#define __zvmhuluians				__builtin_lsp_zvmhuluians
#define __zvmhuluianp				__builtin_lsp_zvmhuluianp
#define __zvmhuluianps				__builtin_lsp_zvmhuluianps
#define __zvmhulsiaa				__builtin_lsp_zvmhulsiaa
#define __zvmhulsiaas				__builtin_lsp_zvmhulsiaas
#define __zvmhulsian				__builtin_lsp_zvmhulsian
#define __zvmhulsians				__builtin_lsp_zvmhulsians
#define __zvmhulsianp				__builtin_lsp_zvmhulsianp
#define __zvmhulsianps				__builtin_lsp_zvmhulsianps
#define __zvmhulsuiaa				__builtin_lsp_zvmhulsuiaa
#define __zvmhulsuiaas				__builtin_lsp_zvmhulsuiaas
#define __zvmhulsuian				__builtin_lsp_zvmhulsuian
#define __zvmhulsuians				__builtin_lsp_zvmhulsuians
#define __zvmhulsuianp				__builtin_lsp_zvmhulsuianp
#define __zvmhulsuianps				__builtin_lsp_zvmhulsuianps
#define __zvmhulsfaas				__builtin_lsp_zvmhulsfaas
#define __zvmhulsfraas				__builtin_lsp_zvmhulsfraas
#define __zvmhulsfans				__builtin_lsp_zvmhulsfans
#define __zvmhulsfrans				__builtin_lsp_zvmhulsfrans
#define __zvmhulsfanps				__builtin_lsp_zvmhulsfanps
#define __zvmhulsfranps				__builtin_lsp_zvmhulsfranps
#define __zvmhlluiaa				__builtin_lsp_zvmhlluiaa
#define __zvmhlluiaas				__builtin_lsp_zvmhlluiaas
#define __zvmhlluian				__builtin_lsp_zvmhlluian
#define __zvmhlluians				__builtin_lsp_zvmhlluians
#define __zvmhlluianp				__builtin_lsp_zvmhlluianp
#define __zvmhlluianps				__builtin_lsp_zvmhlluianps
#define __zvmhllsiaa				__builtin_lsp_zvmhllsiaa
#define __zvmhllsiaas				__builtin_lsp_zvmhllsiaas
#define __zvmhllsian				__builtin_lsp_zvmhllsian
#define __zvmhllsians				__builtin_lsp_zvmhllsians
#define __zvmhllsianp				__builtin_lsp_zvmhllsianp
#define __zvmhllsianps				__builtin_lsp_zvmhllsianps
#define __zvmhllsuiaa				__builtin_lsp_zvmhllsuiaa
#define __zvmhllsuiaas				__builtin_lsp_zvmhllsuiaas
#define __zvmhllsuian				__builtin_lsp_zvmhllsuian
#define __zvmhllsuians				__builtin_lsp_zvmhllsuians
#define __zvmhllsuianp				__builtin_lsp_zvmhllsuianp
#define __zvmhllsuianps				__builtin_lsp_zvmhllsuianps
#define __zvmhllsfaas				__builtin_lsp_zvmhllsfaas
#define __zvmhllsfraas				__builtin_lsp_zvmhllsfraas
#define __zvmhllsfans				__builtin_lsp_zvmhllsfans
#define __zvmhllsfrans				__builtin_lsp_zvmhllsfrans
#define __zvmhllsfanps				__builtin_lsp_zvmhllsfanps
#define __zvmhllsfranps				__builtin_lsp_zvmhllsfranps
#define __zvmhuuuiaa				__builtin_lsp_zvmhuuuiaa
#define __zvmhuuuiaas				__builtin_lsp_zvmhuuuiaas
#define __zvmhuuuian				__builtin_lsp_zvmhuuuian
#define __zvmhuuuians				__builtin_lsp_zvmhuuuians
#define __zvmhuuuianp				__builtin_lsp_zvmhuuuianp
#define __zvmhuuuianps				__builtin_lsp_zvmhuuuianps
#define __zvmhuusiaa				__builtin_lsp_zvmhuusiaa
#define __zvmhuusiaas				__builtin_lsp_zvmhuusiaas
#define __zvmhuusian				__builtin_lsp_zvmhuusian
#define __zvmhuusians				__builtin_lsp_zvmhuusians
#define __zvmhuusianp				__builtin_lsp_zvmhuusianp
#define __zvmhuusianps				__builtin_lsp_zvmhuusianps
#define __zvmhuusuiaa				__builtin_lsp_zvmhuusuiaa
#define __zvmhuusuiaas				__builtin_lsp_zvmhuusuiaas
#define __zvmhuusuian				__builtin_lsp_zvmhuusuian
#define __zvmhuusuians				__builtin_lsp_zvmhuusuians
#define __zvmhuusuianp				__builtin_lsp_zvmhuusuianp
#define __zvmhuusuianps				__builtin_lsp_zvmhuusuianps
#define __zvmhuusfaas				__builtin_lsp_zvmhuusfaas
#define __zvmhuusfraas				__builtin_lsp_zvmhuusfraas
#define __zvmhuusfans				__builtin_lsp_zvmhuusfans
#define __zvmhuusfrans				__builtin_lsp_zvmhuusfrans
#define __zvmhuusfanps				__builtin_lsp_zvmhuusfanps
#define __zvmhuusfranps				__builtin_lsp_zvmhuusfranps
#define __zvmhxluiaa				__builtin_lsp_zvmhxluiaa
#define __zvmhxluiaas				__builtin_lsp_zvmhxluiaas
#define __zvmhxluian				__builtin_lsp_zvmhxluian
#define __zvmhxluians				__builtin_lsp_zvmhxluians
#define __zvmhxluianp				__builtin_lsp_zvmhxluianp
#define __zvmhxluianps				__builtin_lsp_zvmhxluianps
#define __zvmhxlsiaa				__builtin_lsp_zvmhxlsiaa
#define __zvmhxlsiaas				__builtin_lsp_zvmhxlsiaas
#define __zvmhxlsian				__builtin_lsp_zvmhxlsian
#define __zvmhxlsians				__builtin_lsp_zvmhxlsians
#define __zvmhxlsianp				__builtin_lsp_zvmhxlsianp
#define __zvmhxlsianps				__builtin_lsp_zvmhxlsianps
#define __zvmhxlsuiaa				__builtin_lsp_zvmhxlsuiaa
#define __zvmhxlsuiaas				__builtin_lsp_zvmhxlsuiaas
#define __zvmhxlsuian				__builtin_lsp_zvmhxlsuian
#define __zvmhxlsuians				__builtin_lsp_zvmhxlsuians
#define __zvmhxlsuianp				__builtin_lsp_zvmhxlsuianp
#define __zvmhxlsuianps				__builtin_lsp_zvmhxlsuianps
#define __zvmhxlsfaas				__builtin_lsp_zvmhxlsfaas
#define __zvmhxlsfraas				__builtin_lsp_zvmhxlsfraas
#define __zvmhxlsfans				__builtin_lsp_zvmhxlsfans
#define __zvmhxlsfrans				__builtin_lsp_zvmhxlsfrans
#define __zvmhxlsfanps				__builtin_lsp_zvmhxlsfanps
#define __zvmhxlsfranps				__builtin_lsp_zvmhxlsfranps
#define __zmheui				__builtin_lsp_zmheui
#define __zmhesi				__builtin_lsp_zmhesi
#define __zmhesui				__builtin_lsp_zmhesui
#define __zmheoui				__builtin_lsp_zmheoui
#define __zmheosi				__builtin_lsp_zmheosi
#define __zmheosui				__builtin_lsp_zmheosui
#define __zmhoui				__builtin_lsp_zmhoui
#define __zmhosi				__builtin_lsp_zmhosi
#define __zmhosui				__builtin_lsp_zmhosui
#define __zmhesf				__builtin_lsp_zmhesf
#define __zmhesfr				__builtin_lsp_zmhesfr
#define __zmheosf				__builtin_lsp_zmheosf
#define __zmheosfr				__builtin_lsp_zmheosfr
#define __zmhosf				__builtin_lsp_zmhosf
#define __zmhosfr				__builtin_lsp_zmhosfr
#define __zmheuiaa				__builtin_lsp_zmheuiaa
#define __zmheuiaas				__builtin_lsp_zmheuiaas
#define __zmheuian				__builtin_lsp_zmheuian
#define __zmheuians				__builtin_lsp_zmheuians
#define __zmhesiaa				__builtin_lsp_zmhesiaa
#define __zmhesiaas				__builtin_lsp_zmhesiaas
#define __zmhesian				__builtin_lsp_zmhesian
#define __zmhesians				__builtin_lsp_zmhesians
#define __zmhesuiaa				__builtin_lsp_zmhesuiaa
#define __zmhesuiaas				__builtin_lsp_zmhesuiaas
#define __zmhesuian				__builtin_lsp_zmhesuian
#define __zmhesuians				__builtin_lsp_zmhesuians
#define __zmhesfaas				__builtin_lsp_zmhesfaas
#define __zmhesfraas				__builtin_lsp_zmhesfraas
#define __zmhesfans				__builtin_lsp_zmhesfans
#define __zmhesfrans				__builtin_lsp_zmhesfrans
#define __zmheouiaa				__builtin_lsp_zmheouiaa
#define __zmheouiaas				__builtin_lsp_zmheouiaas
#define __zmheouian				__builtin_lsp_zmheouian
#define __zmheouians				__builtin_lsp_zmheouians
#define __zmheosiaa				__builtin_lsp_zmheosiaa
#define __zmheosiaas				__builtin_lsp_zmheosiaas
#define __zmheosian				__builtin_lsp_zmheosian
#define __zmheosians				__builtin_lsp_zmheosians
#define __zmheosuiaa				__builtin_lsp_zmheosuiaa
#define __zmheosuiaas				__builtin_lsp_zmheosuiaas
#define __zmheosuian				__builtin_lsp_zmheosuian
#define __zmheosuians				__builtin_lsp_zmheosuians
#define __zmheosfaas				__builtin_lsp_zmheosfaas
#define __zmheosfraas				__builtin_lsp_zmheosfraas
#define __zmheosfans				__builtin_lsp_zmheosfans
#define __zmheosfrans				__builtin_lsp_zmheosfrans
#define __zmhouiaa				__builtin_lsp_zmhouiaa
#define __zmhouiaas				__builtin_lsp_zmhouiaas
#define __zmhouian				__builtin_lsp_zmhouian
#define __zmhouians				__builtin_lsp_zmhouians
#define __zmhosiaa				__builtin_lsp_zmhosiaa
#define __zmhosiaas				__builtin_lsp_zmhosiaas
#define __zmhosian				__builtin_lsp_zmhosian
#define __zmhosians				__builtin_lsp_zmhosians
#define __zmhosuiaa				__builtin_lsp_zmhosuiaa
#define __zmhosuiaas				__builtin_lsp_zmhosuiaas
#define __zmhosuian				__builtin_lsp_zmhosuian
#define __zmhosuians				__builtin_lsp_zmhosuians
#define __zmhosfaas				__builtin_lsp_zmhosfaas
#define __zmhosfraas				__builtin_lsp_zmhosfraas
#define __zmhosfans				__builtin_lsp_zmhosfans
#define __zmhosfrans				__builtin_lsp_zmhosfrans
#define __zvmhsih				__builtin_lsp_zvmhsih
#define __zvmhsuih				__builtin_lsp_zvmhsuih
#define __zvmhuih				__builtin_lsp_zvmhuih
#define __zvmhuihs				__builtin_lsp_zvmhuihs
#define __zvmhsihs				__builtin_lsp_zvmhsihs
#define __zvmhsuihs				__builtin_lsp_zvmhsuihs
#define __zvmhsiaah				__builtin_lsp_zvmhsiaah
#define __zvmhsuiaah				__builtin_lsp_zvmhsuiaah
#define __zvmhuiaah				__builtin_lsp_zvmhuiaah
#define __zvmhsianh				__builtin_lsp_zvmhsianh
#define __zvmhsuianh				__builtin_lsp_zvmhsuianh
#define __zvmhuianh				__builtin_lsp_zvmhuianh
#define __zvmhuiaahs				__builtin_lsp_zvmhuiaahs
#define __zvmhuianhs				__builtin_lsp_zvmhuianhs
#define __zvmhsiaahs				__builtin_lsp_zvmhsiaahs
#define __zvmhsianhs				__builtin_lsp_zvmhsianhs
#define __zvmhsuiaahs				__builtin_lsp_zvmhsuiaahs
#define __zvmhsuianhs				__builtin_lsp_zvmhsuianhs
#define __zvmhsfh				__builtin_lsp_zvmhsfh
#define __zvmhsfrh				__builtin_lsp_zvmhsfrh
#define __zvmhsfaahs				__builtin_lsp_zvmhsfaahs
#define __zvmhsfraahs				__builtin_lsp_zvmhsfraahs
#define __zvmhsfanhs				__builtin_lsp_zvmhsfanhs
#define __zvmhsfranhs				__builtin_lsp_zvmhsfranhs
#define __zvdotphaui				__builtin_lsp_zvdotphaui
#define __zvdotphauis				__builtin_lsp_zvdotphauis
#define __zvdotphasi				__builtin_lsp_zvdotphasi
#define __zvdotphasis				__builtin_lsp_zvdotphasis
#define __zvdotphasui				__builtin_lsp_zvdotphasui
#define __zvdotphasuis				__builtin_lsp_zvdotphasuis
#define __zvdotphauiaa				__builtin_lsp_zvdotphauiaa
#define __zvdotphauiaas				__builtin_lsp_zvdotphauiaas
#define __zvdotphauian				__builtin_lsp_zvdotphauian
#define __zvdotphauians				__builtin_lsp_zvdotphauians
#define __zvdotphasiaa				__builtin_lsp_zvdotphasiaa
#define __zvdotphasiaas				__builtin_lsp_zvdotphasiaas
#define __zvdotphasian				__builtin_lsp_zvdotphasian
#define __zvdotphasians				__builtin_lsp_zvdotphasians
#define __zvdotphasuiaa				__builtin_lsp_zvdotphasuiaa
#define __zvdotphasuiaas			__builtin_lsp_zvdotphasuiaas
#define __zvdotphasuian				__builtin_lsp_zvdotphasuian
#define __zvdotphasuians			__builtin_lsp_zvdotphasuians
#define __zvdotphasfs				__builtin_lsp_zvdotphasfs
#define __zvdotphasfrs				__builtin_lsp_zvdotphasfrs
#define __zvdotphxasfs				__builtin_lsp_zvdotphxasfs
#define __zvdotphxasfrs				__builtin_lsp_zvdotphxasfrs
#define __zvdotphasfaas				__builtin_lsp_zvdotphasfaas
#define __zvdotphasfraas			__builtin_lsp_zvdotphasfraas
#define __zvdotphasfans				__builtin_lsp_zvdotphasfans
#define __zvdotphasfrans			__builtin_lsp_zvdotphasfrans
#define __zvdotphxasfaas			__builtin_lsp_zvdotphxasfaas
#define __zvdotphxasfraas			__builtin_lsp_zvdotphxasfraas
#define __zvdotphxasfans			__builtin_lsp_zvdotphxasfans
#define __zvdotphxasfrans			__builtin_lsp_zvdotphxasfrans
#define __zvdotphxaui				__builtin_lsp_zvdotphxaui
#define __zvdotphxauis				__builtin_lsp_zvdotphxauis
#define __zvdotphxasi				__builtin_lsp_zvdotphxasi
#define __zvdotphxasis				__builtin_lsp_zvdotphxasis
#define __zvdotphxasui				__builtin_lsp_zvdotphxasui
#define __zvdotphxasuis				__builtin_lsp_zvdotphxasuis
#define __zvdotphxauiaa				__builtin_lsp_zvdotphxauiaa
#define __zvdotphxauiaas			__builtin_lsp_zvdotphxauiaas
#define __zvdotphxauian				__builtin_lsp_zvdotphxauian
#define __zvdotphxauians			__builtin_lsp_zvdotphxauians
#define __zvdotphxasiaa				__builtin_lsp_zvdotphxasiaa
#define __zvdotphxasiaas			__builtin_lsp_zvdotphxasiaas
#define __zvdotphxasian				__builtin_lsp_zvdotphxasian
#define __zvdotphxasians			__builtin_lsp_zvdotphxasians
#define __zvdotphxasuiaa			__builtin_lsp_zvdotphxasuiaa
#define __zvdotphxasuiaas			__builtin_lsp_zvdotphxasuiaas
#define __zvdotphxasuian			__builtin_lsp_zvdotphxasuian
#define __zvdotphxasuians			__builtin_lsp_zvdotphxasuians
#define __zvdotphsui				__builtin_lsp_zvdotphsui
#define __zvdotphssi				__builtin_lsp_zvdotphssi
#define __zvdotphssui				__builtin_lsp_zvdotphssui
#define __zvdotphsuis				__builtin_lsp_zvdotphsuis
#define __zvdotphssis				__builtin_lsp_zvdotphssis
#define __zvdotphssuis				__builtin_lsp_zvdotphssuis
#define __zvdotphsuiaa				__builtin_lsp_zvdotphsuiaa
#define __zvdotphsuian				__builtin_lsp_zvdotphsuian
#define __zvdotphssiaa				__builtin_lsp_zvdotphssiaa
#define __zvdotphssian				__builtin_lsp_zvdotphssian
#define __zvdotphssuiaa				__builtin_lsp_zvdotphssuiaa
#define __zvdotphssuian				__builtin_lsp_zvdotphssuian
#define __zvdotphsuiaas				__builtin_lsp_zvdotphsuiaas
#define __zvdotphsuians				__builtin_lsp_zvdotphsuians
#define __zvdotphssiaas				__builtin_lsp_zvdotphssiaas
#define __zvdotphssians				__builtin_lsp_zvdotphssians
#define __zvdotphssuiaas			__builtin_lsp_zvdotphssuiaas
#define __zvdotphssuians			__builtin_lsp_zvdotphssuians
#define __zvdotphssfs				__builtin_lsp_zvdotphssfs
#define __zvdotphssfrs				__builtin_lsp_zvdotphssfrs
#define __zvdotphssfaas				__builtin_lsp_zvdotphssfaas
#define __zvdotphssfraas			__builtin_lsp_zvdotphssfraas
#define __zvdotphssfans				__builtin_lsp_zvdotphssfans
#define __zvdotphssfrans			__builtin_lsp_zvdotphssfrans
#define __zmwlsiaa				__builtin_lsp_zmwlsiaa
#define __zmwlsuiaa				__builtin_lsp_zmwlsuiaa
#define __zmwluiaa				__builtin_lsp_zmwluiaa
#define __zmwlsian				__builtin_lsp_zmwlsian
#define __zmwlsuian				__builtin_lsp_zmwlsuian
#define __zmwluian				__builtin_lsp_zmwluian
#define __zmwluis				__builtin_lsp_zmwluis
#define __zmwlsis				__builtin_lsp_zmwlsis
#define __zmwlsuis				__builtin_lsp_zmwlsuis
#define __zmwlsiaas				__builtin_lsp_zmwlsiaas
#define __zmwlsians				__builtin_lsp_zmwlsians
#define __zmwlsuiaas				__builtin_lsp_zmwlsuiaas
#define __zmwlsuians				__builtin_lsp_zmwlsuians
#define __zmwluiaas				__builtin_lsp_zmwluiaas
#define __zmwluians				__builtin_lsp_zmwluians
#define __zmwsf					__builtin_lsp_zmwsf
#define __zmwsfr				__builtin_lsp_zmwsfr
#define __zmwsfaas				__builtin_lsp_zmwsfaas
#define __zmwsfraas				__builtin_lsp_zmwsfraas
#define __zmwsfans				__builtin_lsp_zmwsfans
#define __zmwsfrans				__builtin_lsp_zmwsfrans
#define __zaddwus				__builtin_lsp_zaddwus
#define __zsubfwus				__builtin_lsp_zsubfwus
#define __zaddwss				__builtin_lsp_zaddwss
#define __zsubfwss				__builtin_lsp_zsubfwss
#define __zaddheuw				__builtin_lsp_zaddheuw
#define __zsubfheuw				__builtin_lsp_zsubfheuw
#define __zaddhesw				__builtin_lsp_zaddhesw
#define __zsubfhesw				__builtin_lsp_zsubfhesw
#define __zaddhouw				__builtin_lsp_zaddhouw
#define __zsubfhouw				__builtin_lsp_zsubfhouw
#define __zaddhosw				__builtin_lsp_zaddhosw
#define __zsubfhosw				__builtin_lsp_zsubfhosw
#define __zpkswgshfrs				__builtin_lsp_zpkswgshfrs
#define __zpkswgswfrs				__builtin_lsp_zpkswgswfrs
#define __zvselh				__builtin_lsp_zvselh
#define __zxtrw					__builtin_lsp_zxtrw
#define __zbrminc				__builtin_lsp_zbrminc
#define __zcircinc				__builtin_lsp_zcircinc
#define __lsp4_crd__				__builtin_lsp_lsp4_crd__
#define __lsp4_crd__				__builtin_lsp_lsp4_crd__
#define __lsp4_crd__				__builtin_lsp_lsp4_crd__
#define __lsp4_crd__				__builtin_lsp_lsp4_crd__
#define __lsp4_crd__				__builtin_lsp_lsp4_crd__
#define __zlddx					__builtin_lsp_zlddx
#define __zldd					__builtin_lsp_zldd
#define __zldwx					__builtin_lsp_zldwx
#define __zldw					__builtin_lsp_zldw
#define __zldhx					__builtin_lsp_zldhx
#define __zldh					__builtin_lsp_zldh
#define __zlwgsfdx				__builtin_lsp_zlwgsfdx
#define __zlwgsfd				__builtin_lsp_zlwgsfd
#define __zlwwosdx				__builtin_lsp_zlwwosdx
#define __zlwwosd				__builtin_lsp_zlwwosd
#define __zlwhsplatwdx				__builtin_lsp_zlwhsplatwdx
#define __zlwhsplatwd				__builtin_lsp_zlwhsplatwd
#define __zlwhsplatdx				__builtin_lsp_zlwhsplatdx
#define __zlwhsplatd				__builtin_lsp_zlwhsplatd
#define __zlwhgwsfdx				__builtin_lsp_zlwhgwsfdx
#define __zlwhgwsfd				__builtin_lsp_zlwhgwsfd
#define __zlwhedx				__builtin_lsp_zlwhedx
#define __zlwhed				__builtin_lsp_zlwhed
#define __zlwhosdx				__builtin_lsp_zlwhosdx
#define __zlwhosd				__builtin_lsp_zlwhosd
#define __zlwhoudx				__builtin_lsp_zlwhoudx
#define __zlwhoud				__builtin_lsp_zlwhoud
#define __zlwhx					__builtin_lsp_zlwhx
#define __zlwh					__builtin_lsp_zlwh
#define __zlwwx					__builtin_lsp_zlwwx
#define __zlww					__builtin_lsp_zlww
#define __zlhgwsfx				__builtin_lsp_zlhgwsfx
#define __zlhgwsf				__builtin_lsp_zlhgwsf
#define __zlhhsplatx				__builtin_lsp_zlhhsplatx
#define __zlhhsplat				__builtin_lsp_zlhhsplat
#define __zstddx				__builtin_lsp_zstddx
#define __zstdd					__builtin_lsp_zstdd
#define __zstdwx				__builtin_lsp_zstdwx
#define __zstdw					__builtin_lsp_zstdw
#define __zstdhx				__builtin_lsp_zstdhx
#define __zstdh					__builtin_lsp_zstdh
#define __zstwhedx				__builtin_lsp_zstwhedx
#define __zstwhed				__builtin_lsp_zstwhed
#define __zstwhodx				__builtin_lsp_zstwhodx
#define __zstwhod				__builtin_lsp_zstwhod
#define __zlhhex				__builtin_lsp_zlhhex
#define __zlhhe					__builtin_lsp_zlhhe
#define __zlhhosx				__builtin_lsp_zlhhosx
#define __zlhhos				__builtin_lsp_zlhhos
#define __zlhhoux				__builtin_lsp_zlhhoux
#define __zlhhou				__builtin_lsp_zlhhou
#define __zsthex				__builtin_lsp_zsthex
#define __zsthe					__builtin_lsp_zsthe
#define __zsthox				__builtin_lsp_zsthox
#define __zstho					__builtin_lsp_zstho
#define __zstwhx				__builtin_lsp_zstwhx
#define __zstwh					__builtin_lsp_zstwh
#define __zstwwx				__builtin_lsp_zstwwx
#define __zstww					__builtin_lsp_zstww
#define __zlddmx				__builtin_lsp_zlddmx
#define __zlddu					__builtin_lsp_zlddu
#define __zldwmx				__builtin_lsp_zldwmx
#define __zldwu					__builtin_lsp_zldwu
#define __zldhmx				__builtin_lsp_zldhmx
#define __zldhu					__builtin_lsp_zldhu
#define __zlwgsfdmx				__builtin_lsp_zlwgsfdmx
#define __zlwgsfdu				__builtin_lsp_zlwgsfdu
#define __zlwwosdmx				__builtin_lsp_zlwwosdmx
#define __zlwwosdu				__builtin_lsp_zlwwosdu
#define __zlwhsplatwdmx				__builtin_lsp_zlwhsplatwdmx
#define __zlwhsplatwdu				__builtin_lsp_zlwhsplatwdu
#define __zlwhsplatdmx				__builtin_lsp_zlwhsplatdmx
#define __zlwhsplatdu				__builtin_lsp_zlwhsplatdu
#define __zlwhgwsfdmx				__builtin_lsp_zlwhgwsfdmx
#define __zlwhgwsfdu				__builtin_lsp_zlwhgwsfdu
#define __zlwhedmx				__builtin_lsp_zlwhedmx
#define __zlwhedu				__builtin_lsp_zlwhedu
#define __zlwhosdmx				__builtin_lsp_zlwhosdmx
#define __zlwhosdu				__builtin_lsp_zlwhosdu
#define __zlwhoudmx				__builtin_lsp_zlwhoudmx
#define __zlwhoudu				__builtin_lsp_zlwhoudu
#define __zlwhmx				__builtin_lsp_zlwhmx
#define __zlwhu					__builtin_lsp_zlwhu
#define __zlwwmx				__builtin_lsp_zlwwmx
#define __zlwwu					__builtin_lsp_zlwwu
#define __zlhgwsfmx				__builtin_lsp_zlhgwsfmx
#define __zlhgwsfu				__builtin_lsp_zlhgwsfu
#define __zlhhsplatmx				__builtin_lsp_zlhhsplatmx
#define __zlhhsplatu				__builtin_lsp_zlhhsplatu
#define __zlhhemx				__builtin_lsp_zlhhemx
#define __zlhheu				__builtin_lsp_zlhheu
#define __zlhhosmx				__builtin_lsp_zlhhosmx
#define __zlhhosu				__builtin_lsp_zlhhosu
#define __zlhhoumx				__builtin_lsp_zlhhoumx
#define __zlhhouu				__builtin_lsp_zlhhouu
#define __zstddmx				__builtin_lsp_zstddmx
#define __zstddu				__builtin_lsp_zstddu
#define __zstdwmx				__builtin_lsp_zstdwmx
#define __zstdwu				__builtin_lsp_zstdwu
#define __zstdhmx				__builtin_lsp_zstdhmx
#define __zstdhu				__builtin_lsp_zstdhu
#define __zstwhedmx				__builtin_lsp_zstwhedmx
#define __zstwhedu				__builtin_lsp_zstwhedu
#define __zstwhodmx				__builtin_lsp_zstwhodmx
#define __zstwhodu				__builtin_lsp_zstwhodu
#define __zsthemx				__builtin_lsp_zsthemx
#define __zstheu				__builtin_lsp_zstheu
#define __zsthomx				__builtin_lsp_zsthomx
#define __zsthou				__builtin_lsp_zsthou
#define __zstwhmx				__builtin_lsp_zstwhmx
#define __zstwhu				__builtin_lsp_zstwhu
#define __zstwwmx				__builtin_lsp_zstwwmx
#define __zstwwu				__builtin_lsp_zstwwu

///////////////////////////////////////////////////////////////////////////////

#undef __ARGS
#undef __ATTRIBUTE

#ifdef __cplusplus
}
#endif

#endif /* defined(__ppc) && defined(__LSP_APU__) */

#endif /* _LSP_H */
